/*
 * gtkmonop - the gtk+ monopoly simulator
 * Copyright (C) 2001 Erik Bourget
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>

#include "structs.h"
#include "stub.h"
#include "client.h"
#include "xpms.h"
#include "misc_functions.h"
#include "assetbox.h"

#define NODE_KEY "NODE_KEY"
#define NAME_KEY "NAME_KEY"

GtkWidget *grouptree[10][10];
GtkWidget *frame[10];
GtkWidget *tree[10];
GtkWidget *astree[40];
extern estate *estates[];
GdkPixmap *deed[9];

int filled = 0;

/* local function prototypes */
void fill_assetbox(GtkWidget *);
void tree_add_widget(GtkWidget *, GtkWidget *, int);
void tree_remove_widget(GtkWidget *, int);
gboolean show_asset_popup (GtkWidget *widget, GdkEventButton *event, gpointer data);
GtkWidget *new_asset_popup(estate *);
void on_tradeadd_activated (GtkWidget *, trade *);
void on_mortgage_activated (GtkWidget *widget, estate *e);
void on_unmortgage_activated (GtkWidget *widget, estate *e);
void on_buyhouse_activated (GtkWidget *widget, estate *e);
void on_sellhouse_activated (GtkWidget *widget, estate *e);

/* external function bodies */
void update_assetbox(int eid, player *from, player *to) 
{
	extern estate *estates[];
	extern int gamestarted;
	extern GtkWidget *mainwin;
	GtkWidget *assetbox;
	estate *e = estates[eid];

	assetbox = gtk_object_get_data(GTK_OBJECT(mainwin), "assetbox");
	
	if(!filled && gamestarted)
		fill_assetbox(assetbox);
	
	/* update the sucker */

	if(from)
		tree_remove_widget(astree[eid], from->playernum);		
	if(to)
		tree_add_widget(astree[eid], grouptree[e->group][to->playernum], 
				to->playernum);
}

/* local function bodies */
void fill_assetbox(GtkWidget *assetbox) 
{
	extern estate *estates[];
	int i, j, max_group_id;
	GtkWidget *temp;
	char *colors[9];
	GdkBitmap *mask[9];

	for(i = 0; i < 9; i++) {
		colors[i] = malloc(64);
		switch(i) {
		case 0:
			strcpy(colors[i], "Miscellaneous");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, graydeed_xpm);
			break;
		case 1:
			strcpy(colors[i], "Dark Purple");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, darkpurpledeed_xpm);
			break;
		case 2:
			strcpy(colors[i], "Light Blue");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, lightbluedeed_xpm);
			break;
		case 3:
			strcpy(colors[i], "Light Purple");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, lightpurpledeed_xpm);
			break;
		case 4:
			strcpy(colors[i], "Orange");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, orangedeed_xpm);
			break;
		case 5:
			strcpy(colors[i], "Red");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, reddeed_xpm);
			break;
		case 6:
			strcpy(colors[i], "Yellow");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, yellowdeed_xpm);
			break;
		case 7:
			strcpy(colors[i], "Green");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, greendeed_xpm);
			break;
		case 8:
			strcpy(colors[i], "Dark Blue");
			deed[i] = gdk_pixmap_create_from_xpm_d(assetbox->window, &mask[i], NULL, darkbluedeed_xpm);
			break;
		}
	}

	temp = NULL;
	
	for(i = 0; player_of_id(i); i++) {
		frame[i] = gtk_frame_new (_(player_of_id(i)->name));
		gtk_widget_show (frame[i]);
		gtk_box_pack_start (GTK_BOX (assetbox), frame[i], TRUE, TRUE, 0);
		
		tree[i] = gtk_ctree_new (1, 0);
		gtk_clist_set_row_height (GTK_CLIST (tree[i]), 20);
		gtk_clist_set_column_width (GTK_CLIST (tree[i]), 0, 200);
		gtk_clist_set_selection_mode (GTK_CLIST (tree[i]),
					      GTK_SELECTION_EXTENDED);
		gtk_ctree_set_show_stub (GTK_CTREE (tree[i]), FALSE);
		gtk_ctree_set_indent (GTK_CTREE (tree[i]), 16);
		gtk_ctree_set_line_style (GTK_CTREE (tree[i]), GTK_CTREE_LINES_DOTTED);

		gtk_widget_show(tree[i]);
		gtk_container_add (GTK_CONTAINER (frame[i]), tree[i]);
		gtk_signal_connect (GTK_OBJECT (tree[i]), "button_press_event",
				    GTK_SIGNAL_FUNC (show_asset_popup),
				    NULL);
	}
	filled = 1;

	max_group_id = 0;

	for(i = 0; i < 40; i++) {
		if(estates[i]->ownable && (max_group_id < estates[i]->group)) 
			max_group_id = estates[i]->group;
		astree[i] = gtk_label_new(estates[i]->name);
		
		gtk_object_set_data (GTK_OBJECT(astree[i]), NAME_KEY, estates[i]->name);
		gtk_object_set_data (GTK_OBJECT(astree[i]), "estate", estates[i]);
		gtk_widget_show(astree[i]);
	}

	max_group_id = 8;
	
	for(i = 0; player_of_id(i); i++) {
		for(j = 0; j <= max_group_id; j++) {
			grouptree[j][i] = gtk_label_new(colors[j]);
			gtk_object_set_data (GTK_OBJECT(grouptree[j][i]), NAME_KEY, colors[j]);
			tree_add_widget(grouptree[j][i], tree[i], i);
		}
	}
}

void tree_add_widget (GtkWidget *widget, GtkWidget *parent, int i) 
{
	gchar *text[1];
	GtkCTreeNode *node, *parent_node;
	estate *e = gtk_object_get_data(GTK_OBJECT(widget), "estate");
	
	parent_node = parent ? 
		((GtkCTreeNode *)gtk_object_get_data(GTK_OBJECT(parent), NODE_KEY)) :
		NULL;
	
	
	text[0] = gtk_object_get_data(GTK_OBJECT(widget), NAME_KEY);
	if(e) { 
		node = gtk_ctree_insert_node (GTK_CTREE (tree[i]),
					      parent_node, NULL, text, 5,
					      deed[e->group], NULL, deed[e->group], NULL,
					      FALSE, TRUE);
	}
	else node =  gtk_ctree_insert_node (GTK_CTREE (tree[i]),
					    parent_node, NULL, text, 5,
					    NULL, NULL, NULL, NULL,
					    FALSE, TRUE);
	gtk_object_set_data (GTK_OBJECT(widget), NODE_KEY, node);
	
	gtk_ctree_node_set_row_data (GTK_CTREE(tree[i]), node, gtk_object_get_data(GTK_OBJECT(widget), "estate"));
}

gboolean show_asset_popup (GtkWidget *widget, GdkEventButton *event, gpointer data) 
{
	GtkWidget *popup;
	estate *e;
	int row, col;
	
	if(event->button == 3) { /* right click */
		if (gtk_clist_get_selection_info
		    (GTK_CLIST (widget), event->x, event->y, &row, &col) < 0)
			return FALSE;
		gtk_clist_unselect_all (GTK_CLIST (widget));
		gtk_clist_select_row (GTK_CLIST (widget), row, 0);
		if(gtk_clist_get_row_data(GTK_CLIST(widget), row)) {
			if ((e = gtk_clist_get_row_data(GTK_CLIST(widget), row))) {
				popup = new_asset_popup(e);
				gtk_widget_show(popup);
				gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL, event->button, event->time);
				return TRUE;
			}
		}
	}
	return FALSE;
}

GtkWidget *new_asset_popup(estate *e) 
{
	extern trade *trades;
	extern gmonopprefs *prefs;
	
	trade *step;
	GtkWidget *popup;
	GtkWidget *item;
	GtkAccelGroup *popup_accels;
	int tradeadded = 0;
	char str[1024];
	
	popup = gtk_menu_new ();
	popup_accels = gtk_menu_ensure_uline_accel_group(GTK_MENU(popup));

	for(step = trades; step; step = step->next) {
		if((step->p[0] == e->owner) || (step->p[1] == e->owner)) {
			if(step->included[e->id])
				sprintf(str, "Remove %s from trade with %d. %s", e->name, 
					((step->p[0]->playernum == prefs->me) ? step->p[1]->playernum : step->p[0]->playernum),
					((step->p[0]->playernum == prefs->me) ? step->p[1]->name : step->p[0]->name));
			else
				sprintf(str, "Add %s to trade with %d. %s", e->name, 
					((step->p[0]->playernum == prefs->me) ? step->p[1]->playernum : step->p[0]->playernum),
					((step->p[0]->playernum == prefs->me) ? step->p[1]->name : step->p[0]->name));

			item = gtk_menu_item_new_with_label(str);
			gtk_widget_show(item);
			gtk_container_add(GTK_CONTAINER(popup), item);

			gtk_object_set_data(GTK_OBJECT(item), "estate", e);
			
			gtk_signal_connect (GTK_OBJECT(item), "activate",
					    GTK_SIGNAL_FUNC (on_tradeadd_activated),
					    step);
			tradeadded = 1;
		}
	}

	if(tradeadded && (e->mortgageable || e->unmortgageable || e->canbuyhouses || e->cansellhouses)) { /* add separator */
		item = gtk_menu_item_new();
		gtk_widget_show(item);
		gtk_container_add(GTK_CONTAINER(popup), item);
	}
	
	if(e->mortgageable) {
		sprintf(str, "Mortgage %s", e->name);
		item = gtk_menu_item_new_with_label(str);
		gtk_widget_show(item);
		gtk_container_add(GTK_CONTAINER(popup), item);
		gtk_signal_connect (GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC (on_mortgage_activated),
				    e);
	}
	if(e->unmortgageable) {
		sprintf(str, "Unmortgage %s", e->name);
		item = gtk_menu_item_new_with_label(str);
		gtk_widget_show(item);
		gtk_container_add(GTK_CONTAINER(popup), item);
		gtk_signal_connect (GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC (on_unmortgage_activated),
				    e);
	}
	if(e->canbuyhouses) {
		sprintf(str, "Build a house on %s ($%d)", e->name, e->houseprice);
		item = gtk_menu_item_new_with_label(str);
		gtk_widget_show(item);
		gtk_container_add(GTK_CONTAINER(popup), item);
		gtk_signal_connect (GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC (on_buyhouse_activated),
				    e);
	}
	if(e->cansellhouses) {
		sprintf(str, "Sell a house on %s", e->name);
		item = gtk_menu_item_new_with_label(str);
		gtk_widget_show(item);
		gtk_container_add(GTK_CONTAINER(popup), item);
		gtk_signal_connect (GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC (on_sellhouse_activated),
				    e);
	}
	

	return popup;
}

void on_mortgage_activated (GtkWidget *widget, estate *e) 
{
	char *buf = (char *)malloc(20);
	
	sprintf(buf, "%d", e->id);
	client_outmessage(OUT_MORTGAGE, buf);
	free(buf);
}

void on_unmortgage_activated (GtkWidget *widget, estate *e) 
{
	char *buf = (char *)malloc(20);
	
	sprintf(buf, "%d", e->id);
	client_outmessage(OUT_UNMORTGAGE, buf);
	free(buf);
}

void on_buyhouse_activated (GtkWidget *widget, estate *e) 
{
	char *buf = (char *)malloc(20);
	
	sprintf(buf, "%d", e->id);
	client_outmessage(OUT_BUYHOUSE, buf);
	free(buf);
}

void on_sellhouse_activated (GtkWidget *widget, estate *e) 
{
	char *buf = (char *)malloc(20);
	
	sprintf(buf, "%d", e->id);
	client_outmessage(OUT_SELLHOUSE, buf);
	free(buf);
}

void on_tradeadd_activated (GtkWidget *widget, trade *t) 
{
	estate *e;
	char *buf = (char *)malloc(20);
	
	e = gtk_object_get_data(GTK_OBJECT(widget), "estate");
	t->included[e->id] = !t->included[e->id];
	sprintf(buf, "%d:%d", t->id, e->id);
	client_outmessage(OUT_TRADEESTATE, buf);
	free(buf);
}


void tree_remove_widget(GtkWidget *widget, int i) 
{
	GtkCTreeNode *node;
	
	node = gtk_object_get_data (GTK_OBJECT (widget), NODE_KEY);
	if(node == NULL)
		return;
	gtk_ctree_remove_node (GTK_CTREE (tree[i]), GTK_CTREE_NODE (node));
}
